#!/usr/bin/env node
"use strict";
const config = require('./.config.json');

const Irc = require("matrix-org-irc");
const Mysql = require("mysql");

require('dotenv').config({path: '/srv/http/w0bm/.env'});

let db = Mysql.createPool({
    socketPath: '/run/mysqld/mysqld.sock',
    user: process.env.DB_USERNAME,
    password: process.env.DB_PASSWORD,
    database: process.env.DB_DATABASE,
    timezone: "utc"
});

let irc = {};

irc.client = new Irc.Client(config.servers.n0xy.host, config.servers.n0xy.nick, {
    port: config.servers.n0xy.port,
    secure: config.servers.n0xy.secure,
    userName: config.servers.n0xy.userName,
    realName: config.servers.n0xy.realName,
    sasl: config.servers.n0xy.sasl,
    debug: false,
    debugRaw: false,
    selfSigned: false,
    autoConnect: true,
    password: config.servers.n0xy.srvPassword,
    channels: ["#w0bm", "#f0ck"]
});

if (process.env.NODE_ENV != 'production') {
    const repl = require("repl").start("> ");
    repl.context.i = irc;
    repl.context.c = irc.client;
    repl.context.w = (msg) => irc.client.say("#w0bm", msg);
    repl.context.s = (cmd, msg) => irc.client.send(cmd, msg);

    Object.defineProperty(repl.context, "q", {get:process.exit});
}

const handleChatCommands = require("./chatCommands")(irc, db);

const shiftRegex = /^(\.?\w+:?)(?:\s+(.+))?/;
const w0bmregex = /(?:\b(\w+)\.)?\bw0bm\.com\/(?:\w+\/){0,3}([^ '\/@`´]+\.(webm)|\d+)/i;
const capsmsg = [
    "Captain capslock did not approve this!",
    "CAPS",
    "caps lock. unleash the mother fucking fury",
    "DON'T YOU TYPE AT ME IN THAT TONE OF VOICE.",
    "Suddenly, CAPS LOCK",
    "OH, so you wanna argue? BRING IT. I got my CAPS LOCK ON",
    "WELCOME TO THE INTERNET"
];

const haitext = ["ohaio", "ohayou", "ohayo", "ohaiou", "ohai", "nabend", "morgen", "guten tag", "hallo", "hi", "hai", "hey", "moin", "hellau", "hello", "hallu", "yo", "servus", "tag", "tach", "alaaf", "alaf", "As-Salamu 'alaikum wa Rahmatullahi wa Barakatuhu"];

const pr0text = [
    "pr0gramm.com existiert nicht!",
    "endlich oc von deiner fotze?",
    "Pfui Daibel",
    "kommandozeile war vorher da",
    "keiner von uns",
    "Falsches Grau",
    "das reicht uns nicht",
    "Zapalot nochmal!",
    "meinten sie reddit, 9gag oder fagbook?",
    "lustig weil pr0gramm",
    "lustig weil fett",
    "marina die huuure",
    "har har har kefer",
    "Ach du scheiße. Da drückste Minus.",
    "0815 wie fick",
    "stumpf ist trumpf",
    "f man weiß ja nie",
    "lass das nicht die repost polizei sehen!"
];

irc.client.addListener("error", message => console.log("error: ", message));

irc.client.addListener("message", (from, to, message, raw) => {
    const client = irc.client;
    let pm;
    if (pm = !/^#/.test(to)) {
        if (client.opt.debug) console.log("setting to from:", to, "to:", from, "cause its a PM");
        to = from;
    }
    if (client.opt.debug) console.log("from:", from, "to:", to, "message:", message);
    if (shiftRegex.test(message)) {
        let cmd = message.match(shiftRegex);
        let args = (cmd[2] || "").trim();
        cmd = cmd[1];

        if (handleChatCommands({
            from: from,
            to: to,
            message: message,
            cmd: cmd,
            args: args
        })) {
            if (client.opt.debug)
                console.log("cmd:", cmd, "arg:", args);
            return;
        }
    }

    /*if (/pr0gramm\.com/.test(message) && to == "#w0bm") {
        client.say(to, pr0text[Math.random()*pr0text.length|0]);
        return;
    }*/

    if (message.toUpperCase() == message && message.trim().split("").filter(c=>/[A-Z]/.test(c)).length > 5) {
        client.say(to, capsmsg[Math.random()*capsmsg.length|0]);
    }

    if (w0bmregex.test(message)) {
        var link = message.match(w0bmregex);
        if (link) {
            var id = link[2];
            if (id.length > 20) {
                client.say(to, "fuck you");
                return;
            }
            var webm = link[3];
            db.getConnection((err, con) => {
                if (err) {
                    client.say(to, "sorry.. there seems to be a database problem right now D:");
                    con.release();
                    return;
                }
                con.query(
                    "select v.id, v.deleted_at, u.username, c.name as category, v.videotitle, v.interpret, v.songtitle, v.imgsource,"+
                    " (select 1 from taggable_taggables t where t.taggable_id = v.id and t.tag_id = 1) as sfw,"+
                    " (select count(cm.id) from comments cm where cm.video_id = v.id and cm.deleted_at is NULL) as comments"+
                    " from videos v, users u, categories c where v.user_id = u.id and v.category_id = c.id"+
                    " and " + (webm ? "v.file" : "v.id") + " = '" + id + "'", (err, rows, fields) => {
                    if (err) {
                        client.say(to, "sorry.. there seems to be an sql query problem D:");
                        client.say(to, "" + err);
                        con.release();
                        return;
                    }
                    var vid = rows[0];
                    if (!vid) {
                        client.say(to, from + ": konnte kein video finden D:");
                        con.release();
                        return;
                    }
                    var txt = [];
                    if (vid.deleted_at) txt.push("\x02gelöscht\x0f");
                    if (webm) txt.push("Link: \x02https://w0bm.com/" + vid.id + "\x0f");
                    txt.push(vid["sfw"] ? "\x039SFW\x0f" : "\x034NSFW\x0f");
                    ["videotitle", "interpret", "songtitle", "category", "username", "comments"].forEach((v, i) => {
                        if (vid[v]) {
                            txt.push(
                                ["Video Title", "Artist", "Song Title", "Category", "Uploader", "Comments"][i] + ": "
                                +"\x02" + vid[v] + "\x0f"
                            );
                        }
                    });

                    client.say(to, txt.join(" | "));
                    con.release();
                });
            });
            return;
        }
    }

    if (haitext.includes(message.toLowerCase())) {
            client.say(to, haitext[Math.random()*haitext.length|0] + " " + from + "!");
            return;
    }

    if (/^a+[yi][yiou]*$/.test(message)) {
        client.say(to, "ayy lmao");
        return;
    }

    if (/.+\?\?$/.test(message)) {
        let ye = Math.random() * 2 | 0;
        client.say(to, "[" + (ye?"x":" ") + "] ja  [" + (ye?" ":"x") + "] nein | " + message);
        return;
    }
});

